#include "globals.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "remote.h"

#define MFG_ID_SST          0xBF

#define SST_ID_SST39VF512   0xD4
#define SST_ID_SST39VF010   0xD5
#define SST_ID_SST39VF020   0xD6
#define SST_ID_SST39VF040   0xD7

static void querynand(int& flashsize)
{
	sendbyte(CMD_QUERYNAND);
	uint8_t manufacturer = recvbyte();
	uint8_t device = recvbyte();

	if (manufacturer != MFG_ID_SST)
		error("unknown flash manufacturer reported");

	const char* flashdevice;
	switch (device)
	{
        case SST_ID_SST39VF512:
        	flashsize = 0x10000;
        	flashdevice = "SST SST39VF512";
        	break;

        case SST_ID_SST39VF010:
        	flashsize = 0x20000;
        	flashdevice = "SST SST39VF010";
        	break;

        case SST_ID_SST39VF020:
        	flashsize = 0x40000;
        	flashdevice = "SST SST39VF020";
        	break;

        case SST_ID_SST39VF040:
        	flashsize = 0x80000;
        	flashdevice = "SST SST39VF040";
        	break;

        default:
        	error("unknown flash type reported");
	}
	printf("Detected flash device %s: %d bytes\n", flashdevice, flashsize);

}

static void status(int current, int total, int max, char c)
{
	putchar('\r');
	int i = 0;
	while (i < current)
	{
		putchar('+');
		i++;
	}
	if (i < max)
	{
		putchar(c);
		i++;
		while (i < total)
		{
			putchar('-');
			i++;
		}
		while (i < max)
		{
			putchar('.');
			i++;
		}
	}
	fflush(stdout);

}

void cmd_readnand(char* const* argv)
{
	if (!argv[0] || argv[1])
		error("syntax: readnand <filename>");


}

void cmd_writenand(char* const* argv)
{
	if (!argv[0] || argv[1])
		error("syntax: writenand <filename>");

	FILE* fp = fopen(argv[0], "rb");
	if (!fp)
		error("unable to open input file '%s'", argv[0]);

	remote_connect();

	int flashsize;
	querynand(flashsize);

	int filesize;
	{
		struct stat st;
		fstat(fileno(fp), &st);
		filesize = st.st_size;
	}
	if (filesize > flashsize)
	{
		warning("WARNING: file size too big to fit in flash --- truncating!");
		filesize = flashsize;
	}
	int sectors = (filesize + (SECTORSIZE - 1)) / SECTORSIZE;

	printf("File size is %d bytes (%d sectors of 4096 bytes each)\n",
			filesize, sectors);

	for (int sector = 0; sector < sectors; sector++)
	{
again:
		status(sector, sectors, flashsize/SECTORSIZE, 'W');

		uint8_t buffer[SECTORSIZE];
		memset(buffer, 0, SECTORSIZE);
		fseek(fp, sector*SECTORSIZE, SEEK_SET);
		int length = fread(buffer, 1, SECTORSIZE, fp);

		sendbyte(CMD_WRITENAND);
		sendbyte(sector);
		for (int i = 0; i < SECTORSIZE; i++)
			sendbyte(buffer[i]);
		sendquad(CRC32(buffer, SECTORSIZE));

		switch (recvbyte())
		{
			case RSP_SUCCESS:
				break;

			case RSP_RETRY:
				goto again;

			default:
				putchar('\n');
				error("Programming failed");
		}
	}
	status(sectors, sectors, flashsize/SECTORSIZE, '.');
	putchar('\n');
}
