#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <getopt.h>
#include "globals.h"

static int argc;
static char* const* argv;

const char* SerialPort = "/dev/ttyS0";
bool Verbose = false;

static void show_help()
{
	printf(
		"cytool: Cybiko boot loader client v%s.\n"
		"(C) 2010 David Given.\n"
		"Usage: cytool [<options>] <command...>\n"
		"Options:\n"
		"   -h         Displays this message\n"
		"   -p <port>  Specify serial port\n"
		"Commands:\n"
		"   readnand   Reads the NAND (program) flash\n"
		"   writenand  Reprograms NAND (program) flash\n"
		"   term       Start simple serial terminal\n"
		"WARNING. Use this program at your own risk. The author accepts no responsibility\n"
		"    for any damage this program may do to your hardware. You have been warned!\n",
		VERSION
	);
}

static void parse_options()
{
	for (;;)
	{
		int c = getopt(argc, argv,
			":hp:");
		switch (c)
		{
			case -1:
				argv += optind;
				return;

			case 'h':
				show_help();
				exit(0);

			case 'p':
				SerialPort = optarg;
				break;

			case ':':
				error("missing option argument.");

			default:
				error("try '-h' for a usage summary.");
		}
	}
}

int main(int _argc, char* const* _argv)
{
	argc = _argc;
	argv = _argv;
	parse_options();

	const char* cmd = argv[0];
	if (!cmd)
		error("nothing to do! Try '-h' for a usage summary.");

	argv++;
	if (strcmp(cmd, "term") == 0)
		dodgyterm();
	else if (strcmp(cmd, "readnand") == 0)
		cmd_readnand(argv);
	else if (strcmp(cmd, "writenand") == 0)
		cmd_writenand(argv);
	else
	{
		show_help();
		exit(1);
	}
}
