#include "globals.h"
#include "remote.h"

using std::string;

const int PACKETSIZE = 512;

extern "C" const char remoteprogram_data[];
extern "C" unsigned int remoteprogram_size;

static string readline()
{
	string s;

	for (;;)
	{
		int c = recvbyte();
		if (c == '\r')
			continue;
		if (c == '\n')
			break;

		s += (char) c;
	}

	return s;
}

static string readlineb()
{
	for (;;)
	{
		string s = readline();
		if (s != "")
			return s;
	}
}

static void sendstring(const char* s)
{
	for (;;)
	{
		char c = *s++;
		if (!c)
			break;
		sendbyte(c);
	}
}

static void sendblock(const uint8_t* ptr, int size)
{
	while (size--)
		sendbyte(*ptr++);
}

void remote_connect()
{
	printf("Waiting for device reset...\n");

	serial_connect();

	for (;;)
	{
		string s = readline();
		if (s == "Preparing to load CyOS")
			break;
	}

	//72,63,76,20,2f,64,65,66,61,75,6c,74,2f,73,65,74,74,69,6e,67,73,2e,69,6e,69,20,38,35,34,0a,3a,39,
	//rcv /default/

	{
		char buffer[32];
		sprintf(buffer, "rcv remote 0x%x\r\n", remoteprogram_size);
		sendstring(buffer);
	}

	string s = readlineb();
	if (!BeginsWith(s, "receiving file "))
		error("didn't understand '%s' from CyBoot", s.c_str());

	printf("Sending remote agent... ");
	for (;;)
	{
		string s = readlineb();
//		printf("<%s>\n", s.c_str());
		if (s == "done")
			break;
		if (!BeginsWith(s, "send"))
			error("didn't understand '%s' from CyBoot", s.c_str());

		int packet = strtoul(s.c_str() + 4, NULL, 10);
		int offset = packet << 9;
		printf("#");
		fflush(stdout);

		int length = remoteprogram_size - offset;
		if (length > PACKETSIZE)
			length = PACKETSIZE;
//		printf("length=%d\n", length);

		uint8_t buffer[length + 5];
		buffer[0] = 'C';
		buffer[1] = packet & 0xff;
		buffer[2] = packet >> 8;
		buffer[3] = length & 0xff;
		buffer[4] = length >> 8;

		memcpy(&buffer[5], remoteprogram_data + offset, length);
		sendblock(buffer, length + 5);

		uint32_t crc = CRC32(buffer+1, length+4);

		sendbyte((crc >> 0) & 0xff);
		sendbyte((crc >> 8) & 0xff);
		sendbyte((crc >> 16) & 0xff);
		sendbyte((crc >> 24) & 0xff);
	}
	printf("\n");

	for (;;)
	{
		string s = readline();
		//printf("<%s>\n", s.c_str());
		if (s == BANNER)
			break;
	}
}
