#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <termios.h>
#include <sys/poll.h>
#include <string.h>
#include <errno.h>
#include <limits.h>
#include "globals.h"

static int fd;
static struct termios serialterm;

void serial_connect()
{
	fd = open(SerialPort, O_RDWR | O_NOCTTY | O_NDELAY);
	if (fd == -1)
		error("Failed to open serial port.");

	tcgetattr(fd, &serialterm);
	serialterm.c_cflag = CS8 | CLOCAL | CREAD;
	serialterm.c_lflag = 0;
	serialterm.c_oflag = 0;
	serialterm.c_iflag = IGNPAR;

	cfsetspeed(&serialterm, B57600);
	int i = tcsetattr(fd, TCSADRAIN, &serialterm);
	if (i == -1)
		error("Failed to set up serial port.");
}

void sendbyte(uint8_t c)
{
	int i;

#ifdef SPEW_TRACING
	printf("<%02X\n", c);
#endif

	do
	{
		struct pollfd p;
		p.fd = fd;
		p.events = POLLOUT;
		if (poll(&p, 1, INT_MAX) == 0)
			break;

		i = write(fd, &c, 1);
		if (i == -1)
			error("I/O error on write: %s",
				strerror(errno));
	}
	while (i != 1);
}

uint8_t recvbyte()
{
	int i;
	uint8_t b;

	do
	{
		struct pollfd p;
		p.fd = fd;
		p.events = POLLIN;
		if (poll(&p, 1, INT_MAX) == 0)
			break;

		i = read(fd, &b, 1);
		if (i == -1)
			error("I/O error on read: %s",
				strerror(errno));
	}
	while (i != 1);

#ifdef SPEW_TRACING
	printf(">%02X\n", b);
#endif
	return b;
}

void sendquad(uint32_t q)
{
	sendbyte(q >> 0);
	sendbyte(q >> 8);
	sendbyte(q >> 16);
	sendbyte(q >> 24);
}

void dodgyterm()
{
	printf("Serial terminal starting (CTRL+C to quit)\n");

	serial_connect();

	/* Put the console into raw mode. */

	struct termios oldconsole;
	struct termios console;
	tcgetattr(0, &oldconsole);
	console = oldconsole;
	cfmakeraw(&console);
	int i = tcsetattr(0, TCSANOW, &console);
	if (i == -1)
		error("Failed to put console into raw mode.");

	/* Wait for input on either device. */

	for (;;)
	{
		struct pollfd p[2];
		p[0].fd = fd;
		p[0].events = POLLIN;
		p[1].fd = 0;
		p[1].events = POLLIN;

		poll(p, 2, INT_MAX);

		if (p[0].revents)
		{
			uint8_t b = recvbyte();
			write(0, &b, 1);
		}

		if (p[1].revents)
		{
			uint8_t b;
			read(0, &b, 1);
			if (b == 3)
				break;
			sendbyte(b);
		}
	}

	/* Put the console back the way it was. */

	tcsetattr(0, TCSANOW, &oldconsole);
}

