#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdarg.h>
#include <sys/time.h>
#include <time.h>
#include <string>
#include "globals.h"

using std::string;

static struct timeval timezero;

void warning(const char* message, ...)
{
	va_list ap;
	va_start(ap, message);

	fprintf(stderr, "cytool: ");
	vfprintf(stderr, message, ap);
	fprintf(stderr, "\n");

	va_end(ap);
};

void error(const char* message, ...)
{
	va_list ap;
	va_start(ap, message);

	fprintf(stderr, "cytool: ");
	vfprintf(stderr, message, ap);
	fprintf(stderr, "\n");

	va_end(ap);
	exit(1);
};

void verbose(const char* message, ...)
{
	if (Verbose)
	{
		va_list ap;
		va_start(ap, message);
		vprintf(message, ap);
		va_end(ap);
	}
};

void resettimer()
{
	gettimeofday(&timezero, NULL);
};

uint32_t gettime()
{
	struct timeval timenow;
	gettimeofday(&timenow, NULL);

	return ((timenow.tv_sec - timezero.tv_sec) * 1000) +
	       ((timenow.tv_usec - timezero.tv_usec) / 1000);
};

bool BeginsWith(const string& haystack, const string& needle)
{
	return haystack.compare(0, needle.size(), needle) == 0;
}

uint16_t CRC16(const uint8_t* data, int size)
{
    uint32_t crc = 0;

    for (int i=0; i<size; i++)
    	crc = ((crc ^ (*data++)) << 1) | ((crc >> 16) & 1);

    return crc;
}

uint32_t CRC32(const uint8_t* data, int size)
{
	uint32_t crc = 0xffffffff;

	for (int i=0; i<size; i++)
	{
		uint8_t o = (crc & 0xff) ^ *data++;
		crc = (crc >> 8) ^ crctable[o];
	}

	return ~crc;
}


